@@ -5,6 +5,7 @@ package quicktest_test
 import (
 	"bytes"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"strings"
 	"testing"
@@ -54,6 +55,8 @@ type OuterJSON struct {
 	Second []*InnerJSON `json:"Last,omitempty" yaml:"last,omitempty"`
 }
 
+type boolean bool
+
 var checkerTests = []struct {
 	about                 string
 	checker               qt.Checker
@@ -101,6 +104,62 @@ got:
 want:
   ~string "bar"~
 `),
+}, {
+	about:   "Equals: same multiline strings",
+	checker: qt.Equals,
+	got:     "a\nmultiline\nstring",
+	args:    []interface{}{"a\nmultiline\nstring"},
+	expectedNegateFailure: `
+error:
+  unexpected success
+got:
+  "a\nmultiline\nstring"
+want:
+  <same as "got">
+`,
+}, {
+	about:   "Equals: different multi-line strings",
+	checker: qt.Equals,
+	got:     "a\nlong\nmultiline\nstring",
+	args:    []interface{}{"just\na\nlong\nmulti-line\nstring\n"},
+	expectedCheckFailure: fmt.Sprintf(`
+error:
+  values are not equal
+line diff (-got +want):
+%s
+got:
+  "a\nlong\nmultiline\nstring"
+want:
+  "just\na\nlong\nmulti-line\nstring\n"
+`, diff([]string{"a\n", "long\n", "multiline\n", "string"}, []string{"just\n", "a\n", "long\n", "multi-line\n", "string\n", ""})),
+}, {
+	about:   "Equals: different single-line strings ending with newline",
+	checker: qt.Equals,
+	got:     "foo\n",
+	args:    []interface{}{"bar\n"},
+	expectedCheckFailure: `
+error:
+  values are not equal
+got:
+  "foo\n"
+want:
+  "bar\n"
+`,
+}, {
+	about:   "Equals: different strings starting with newline",
+	checker: qt.Equals,
+	got:     "\nfoo",
+	args:    []interface{}{"\nbar"},
+	expectedCheckFailure: fmt.Sprintf(`
+error:
+  values are not equal
+line diff (-got +want):
+%s
+got:
+  "\nfoo"
+want:
+  "\nbar"
+`, diff([]string{"\n", "foo"}, []string{"\n", "bar"})),
 }, {
 	about:   "Equals: different types",
 	checker: qt.Equals,
@@ -184,6 +243,25 @@ got:
 want:
   nil
 `),
+}, {
+	about:   "Equals: different errors with same message",
+	checker: qt.Equals,
+	got: &errTest{
+		msg: "bad wolf",
+	},
+	args: []interface{}{errors.New("bad wolf")},
+	expectedCheckFailure: `
+error:
+  values are not equal
+got type:
+  *quicktest_test.errTest
+want type:
+  *errors.errorString
+got:
+  e"bad wolf"
+want:
+  <same as "got">
+`,
 }, {
 	about:   "Equals: nil struct",
 	checker: qt.Equals,
@@ -197,6 +275,19 @@ got:
 want:
   nil
 `,
+}, {
+	about:   "Equals: different booleans",
+	checker: qt.Equals,
+	got:     true,
+	args:    []interface{}{false},
+	expectedCheckFailure: `
+error:
+  values are not equal
+got:
+  bool(true)
+want:
+  bool(false)
+`,
 }, {
 	about:   "Equals: uncomparable types",
 	checker: qt.Equals,
@@ -1327,12 +1418,12 @@ got:
   []int(nil)
 `,
 }, {
-	about:   "IsNil: error does not guard against nil",
+	about:   "IsNil: nil error-implementing type",
 	checker: qt.IsNil,
 	got:     (*errTest)(nil),
-	expectedNegateFailure: `
+	expectedCheckFailure: `
 error:
-  unexpected success
+  error containing nil value of type *quicktest_test.errTest. See https://golang.org/doc/faq#nil_error
 got:
   e<nil>
 `,
@@ -1342,10 +1433,21 @@ got:
 	got:     42,
 	expectedCheckFailure: `
 error:
-  42 is not nil
+  got non-nil value
 got:
   int(42)
 `,
+}, {
+	about:   "IsNil: error is not nil",
+	checker: qt.IsNil,
+	got:     errBadWolf,
+	expectedCheckFailure: `
+error:
+  got non-nil error
+got:
+  bad wolf
+    file:line
+`,
 }, {
 	about:   "IsNil: too many arguments",
 	checker: qt.IsNil,
@@ -1859,13 +1961,157 @@ got args:
 want args:
   predicate function
 `,
+}, {
+	about:   "IsTrue: success",
+	checker: qt.IsTrue,
+	got:     true,
+	expectedNegateFailure: `
+error:
+  unexpected success
+got:
+  bool(true)
+`,
+}, {
+	about:   "IsTrue: failure",
+	checker: qt.IsTrue,
+	got:     false,
+	expectedCheckFailure: `
+error:
+  value is not true
+got:
+  bool(false)
+`,
+}, {
+	about:   "IsTrue: success with subtype",
+	checker: qt.IsTrue,
+	got:     boolean(true),
+	expectedNegateFailure: `
+error:
+  unexpected success
+got:
+  quicktest_test.boolean(true)
+`,
+}, {
+	about:   "IsTrue: failure with subtype",
+	checker: qt.IsTrue,
+	got:     boolean(false),
+	expectedCheckFailure: `
+error:
+  value is not true
+got:
+  quicktest_test.boolean(false)
+`,
+}, {
+	about:   "IsTrue: nil value",
+	checker: qt.IsTrue,
+	got:     nil,
+	expectedCheckFailure: `
+error:
+  bad check: value does not have a bool underlying type
+value:
+  nil
+`,
+	expectedNegateFailure: `
+error:
+  bad check: value does not have a bool underlying type
+value:
+  nil
+`,
+}, {
+	about:   "IsTrue: non-bool value",
+	checker: qt.IsTrue,
+	got:     42,
+	expectedCheckFailure: `
+error:
+  bad check: value does not have a bool underlying type
+value:
+  int(42)
+`,
+	expectedNegateFailure: `
+error:
+  bad check: value does not have a bool underlying type
+value:
+  int(42)
+`,
+}, {
+	about:   "IsFalse: success",
+	checker: qt.IsFalse,
+	got:     false,
+	expectedNegateFailure: `
+error:
+  unexpected success
+got:
+  bool(false)
+`,
+}, {
+	about:   "IsFalse: failure",
+	checker: qt.IsFalse,
+	got:     true,
+	expectedCheckFailure: `
+error:
+  value is not false
+got:
+  bool(true)
+`,
+}, {
+	about:   "IsFalse: success with subtype",
+	checker: qt.IsFalse,
+	got:     boolean(false),
+	expectedNegateFailure: `
+error:
+  unexpected success
+got:
+  quicktest_test.boolean(false)
+`,
+}, {
+	about:   "IsFalse: failure with subtype",
+	checker: qt.IsFalse,
+	got:     boolean(true),
+	expectedCheckFailure: `
+error:
+  value is not false
+got:
+  quicktest_test.boolean(true)
+`,
+}, {
+	about:   "IsFalse: nil value",
+	checker: qt.IsFalse,
+	got:     nil,
+	expectedCheckFailure: `
+error:
+  bad check: value does not have a bool underlying type
+value:
+  nil
+`,
+	expectedNegateFailure: `
+error:
+  bad check: value does not have a bool underlying type
+value:
+  nil
+`,
+}, {
+	about:   "IsFalse: non-bool value",
+	checker: qt.IsFalse,
+	got:     "bad wolf",
+	expectedCheckFailure: `
+error:
+  bad check: value does not have a bool underlying type
+value:
+  "bad wolf"
+`,
+	expectedNegateFailure: `
+error:
+  bad check: value does not have a bool underlying type
+value:
+  "bad wolf"
+`,
 }, {
 	about:   "Not: success",
 	checker: qt.Not(qt.IsNil),
 	got:     42,
 	expectedNegateFailure: `
 error:
-  42 is not nil
+  got non-nil value
 got:
   int(42)
 `,
@@ -1928,7 +2174,7 @@ want args:
 	expectedNegateFailure: `
 error:
   unexpected success
-got:
+container:
   "hello, world"
 want:
   "world"
@@ -1941,7 +2187,7 @@ want:
 	expectedCheckFailure: `
 error:
   no substring match found
-got:
+container:
   "hello, world"
 want:
   "worlds"
@@ -1954,7 +2200,7 @@ want:
 	expectedNegateFailure: `
 error:
   unexpected success
-got:
+container:
   []string{"a", "b", "c"}
 want:
   "a"
@@ -1970,7 +2216,7 @@ want:
 	expectedNegateFailure: `
 error:
   unexpected success
-got:
+container:
   map[string]string{"a":"d"}
 want:
   "d"
@@ -2299,14 +2545,14 @@ diff (-got +want):
 	checker: qt.JSONEquals,
 	got:     `{"NotThere": `,
 	args:    []interface{}{nil},
-	expectedCheckFailure: tilde2bq(`
+	expectedCheckFailure: fmt.Sprintf(tilde2bq(`
 error:
-  cannot unmarshal obtained contents: unexpected end of JSON input; "{\"NotThere\": "
+  cannot unmarshal obtained contents: %s; "{\"NotThere\": "
 got:
   ~{"NotThere": ~
 want:
   nil
-`),
+`), mustJSONUnmarshalErr(`{"NotThere": `)),
 }, {
 	about:   "JSONEquals cannot marshal expected value",
 	checker: qt.JSONEquals,
@@ -2369,14 +2615,14 @@ error:
 	),
 	got:  "null",
 	args: []interface{}{nil},
-	expectedCheckFailure: `
+	expectedCheckFailure: fmt.Sprintf(`
 error:
-  bad check: cannot unmarshal expected contents: invalid character 'b' looking for beginning of value
-`,
-	expectedNegateFailure: `
+  bad check: cannot unmarshal expected contents: %s
+`, mustJSONUnmarshalErr("bad json")),
+	expectedNegateFailure: fmt.Sprintf(`
 error:
-  bad check: cannot unmarshal expected contents: invalid character 'b' looking for beginning of value
-`,
+  bad check: cannot unmarshal expected contents: %s
+`, mustJSONUnmarshalErr("bad json")),
 }, {
 	about: "CodecEquals with options",
 	checker: qt.CodecEquals(
@@ -2396,12 +2642,6 @@ want:
 `),
 }}
 
-type jsonErrorMarshaler struct{}
-
-func (jsonErrorMarshaler) MarshalJSON() ([]byte, error) {
-	return nil, fmt.Errorf("qt json marshal error")
-}
-
 func TestCheckers(t *testing.T) {
 	for _, test := range checkerTests {
 		checker := qt.WithVerbosity(test.checker, test.verbose)
@@ -2425,6 +2665,21 @@ func diff(x, y interface{}, opts ...cmp.Option) string {
 	return strings.TrimSuffix(qt.Prefixf("  ", "%s", d), "\n")
 }
 
+type jsonErrorMarshaler struct{}
+
+func (jsonErrorMarshaler) MarshalJSON() ([]byte, error) {
+	return nil, fmt.Errorf("qt json marshal error")
+}
+
+func mustJSONUnmarshalErr(s string) error {
+	var v interface{}
+	err := json.Unmarshal([]byte(s), &v)
+	if err == nil {
+		panic("want JSON error, got nil")
+	}
+	return err
+}
+
 func tilde2bq(s string) string {
 	return strings.Replace(s, "~", "`", -1)
 }
